package javax.sql.rowset.spi;

import javax.naming.NotContextException;
import javax.naming.Binding;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import java.util.logging.Level;
import com.sun.rowset.providers.RIOptimisticProvider;
import java.security.AccessControlException;
import sun.reflect.misc.ReflectUtil;
import java.util.Enumeration;
import java.io.InputStream;
import java.util.StringTokenizer;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.security.PrivilegedActionException;
import java.io.FileInputStream;
import java.security.AccessController;
import java.util.PropertyPermission;
import java.security.Permission;
import java.security.PrivilegedAction;
import java.util.Properties;
import java.util.Hashtable;
import java.util.logging.Logger;
import javax.naming.Context;
import java.sql.SQLPermission;

public class SyncFactory
{
    public static final String ROWSET_SYNC_PROVIDER = "rowset.provider.classname";
    public static final String ROWSET_SYNC_VENDOR = "rowset.provider.vendor";
    public static final String ROWSET_SYNC_PROVIDER_VERSION = "rowset.provider.version";
    private static String ROWSET_PROPERTIES;
    private static final SQLPermission SET_SYNCFACTORY_PERMISSION;
    private static Context ic;
    private static volatile Logger rsLogger;
    private static Hashtable<String, SyncProvider> implementations;
    private static String colon;
    private static String strFileSep;
    private static boolean debug;
    private static int providerImplIndex;
    private static boolean lazyJNDICtxRefresh;
    
    private SyncFactory() {
    }
    
    public static synchronized void registerProvider(String s) throws SyncFactoryException {
        ProviderImpl value = new ProviderImpl();
        value.setClassname(s);
        initMapIfNecessary();
        SyncFactory.implementations.put(s, value);
    }
    
    public static SyncFactory getSyncFactory() {
        return SyncFactory.SyncFactoryHolder.factory;
    }
    
    public static synchronized void unregisterProvider(String s) throws SyncFactoryException {
        initMapIfNecessary();
        if (SyncFactory.implementations.containsKey(s)) {
            SyncFactory.implementations.remove(s);
        }
    }
    
    private static synchronized void initMapIfNecessary() throws SyncFactoryException {
        Properties properties = new Properties();
        if (SyncFactory.implementations == null) {
            SyncFactory.implementations = new Hashtable();
            try {
                String rowset_PROPERTIES;
                try {
                    rowset_PROPERTIES = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                        @Override
                        public String run() {
                            return System.getProperty("rowset.properties");
                        }
                    }, null, new Permission[] { new PropertyPermission("rowset.properties", "read") });
                }
                catch (Exception obj) {
                    System.out.println("errorget rowset.properties: " + obj);
                    rowset_PROPERTIES = null;
                }
                if (rowset_PROPERTIES != null) {
                    SyncFactory.ROWSET_PROPERTIES = rowset_PROPERTIES;
                    try (final Object inStream = new FileInputStream(SyncFactory.ROWSET_PROPERTIES)) {
                        properties.load(inStream);
                    }
                    parseProperties(properties);
                }
                SyncFactory.ROWSET_PROPERTIES = "javax" + SyncFactory.strFileSep + "sql" + SyncFactory.strFileSep + "rowset" + SyncFactory.strFileSep + "rowset.properties";
                Thread.currentThread().getContextClassLoader();
                try {
                    AccessController.doPrivileged(()/*  */ -> {
                        InputStream inStream2 = (var_2_FB == null) ? ClassLoader.getSystemResourceAsStream(SyncFactory.ROWSET_PROPERTIES) : var_2_FB.getResourceAsStream(SyncFactory.ROWSET_PROPERTIES);
                        try {
                            if (inStream2 == null) {
                                new SyncFactoryException("Resource " + SyncFactory.ROWSET_PROPERTIES + " not found");
                                throw;
                            }
                            var_0_07.load(inStream2);
                            if (inStream2 != null) {
                                Throwable t3;
                                if (t3 != null) {
                                    try {
                                        inStream2.close();
                                    }
                                    catch (Throwable exception4) {
                                        t3.addSuppressed(exception4);
                                    }
                                }
                                else {
                                    inStream2.close();
                                }
                            }
                        }
                        catch (Throwable t4) {
                            throw t4;
                        }
                        finally {
                            if (inStream2 != null) {
                                Throwable t3;
                                if (t3 != null) {
                                    try {
                                        inStream2.close();
                                    }
                                    catch (Throwable exception5) {
                                        t3.addSuppressed(exception5);
                                    }
                                }
                                else {
                                    inStream2.close();
                                }
                            }
                        }
                        return null;
                    });
                }
                catch (PrivilegedActionException ex) {
                    Exception exception3 = ex.getException();
                    if ((exception3 instanceof SyncFactoryException)) {
                        throw (SyncFactoryException)exception3;
                    }
                    SyncFactoryException o = new SyncFactoryException();
                    o.initCause(ex.getException());
                    throw o;
                }
                parseProperties(properties);
            }
            catch (FileNotFoundException obj2) {
                throw new SyncFactoryException("Cannot locate properties file: " + obj2);
            }
            catch (IOException obj3) {
                throw new SyncFactoryException("IOException: " + obj3);
            }
            properties.clear();
            String s;
            try {
                s = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                    @Override
                    public String run() {
                        return System.getProperty("rowset.provider.classname");
                    }
                }, null, new Permission[] { new PropertyPermission("rowset.provider.classname", "read") });
            }
            catch (Exception ex2) {
                s = null;
            }
            if (s != null) {
                int i = 0;
                if (s.indexOf(SyncFactory.colon) > 0) {
                    for (Object o2 = new StringTokenizer(s, SyncFactory.colon); o2.hasMoreElements(); ++i) {
                        properties.put("rowset.provider.classname." + i, o2.nextToken());
                    }
                }
                else {
                    properties.put("rowset.provider.classname", s);
                }
                parseProperties(properties);
            }
        }
    }
    
    private static void parseProperties(Properties properties) {
        Enumeration<Object> propertyNames = properties.propertyNames();
        while (propertyNames.hasMoreElements()) {
            String s = (String)propertyNames.nextElement();
            int length = s.length();
            if (s.startsWith("rowset.provider.classname")) {
                ProviderImpl value = new ProviderImpl();
                value.setIndex(SyncFactory.providerImplIndex++);
                String[] array;
                if (length == "rowset.provider.classname".length()) {
                    array = getPropertyNames(false);
                }
                else {
                    array = getPropertyNames(true, s.substring(length - 1));
                }
                String property = properties.getProperty(array[0]);
                value.setClassname(property);
                value.setVendor(properties.getProperty(array[1]));
                value.setVersion(properties.getProperty(array[2]));
                SyncFactory.implementations.put(property, value);
            }
            else {
                continue;
            }
        }
    }
    
    private static String[] getPropertyNames(boolean b) {
        return getPropertyNames(b, null);
    }
    
    private static String[] getPropertyNames(boolean b, String str) {
        String str2 = ".";
        String[] array = { "rowset.provider.classname", "rowset.provider.vendor", "rowset.provider.version" };
        if (b) {
            for (int i = 0; i < array.length; ++i) {
                array[i] = array[i] + str2 + str;
            }
            return array;
        }
        return array;
    }
    
    private static void showImpl(ProviderImpl providerImpl) {
        System.out.println("Provider implementation:");
        System.out.println("Classname: " + providerImpl.getClassname());
        System.out.println("Vendor: " + providerImpl.getVendor());
        System.out.println("Version: " + providerImpl.getVersion());
        System.out.println("Impl index: " + providerImpl.getIndex());
    }
    
    public static SyncProvider getInstance(String name) throws SyncFactoryException {
        if (name == null) {
            throw new SyncFactoryException("The providerID cannot be null");
        }
        initMapIfNecessary();
        initJNDIContext();
        if ((ProviderImpl)SyncFactory.implementations.get(name) == null) {
            return new RIOptimisticProvider();
        }
        try {
            ReflectUtil.checkPackageAccess(name);
        }
        catch (AccessControlException cause) {
            SyncFactoryException ex = new SyncFactoryException();
            ex.initCause(cause);
            throw ex;
        }
        try {
            Class<Object> forName = Class.forName(name, true, Thread.currentThread().getContextClassLoader());
            if (forName != null) {
                return (SyncProvider)forName.newInstance();
            }
            return new RIOptimisticProvider();
        }
        catch (IllegalAccessException ex2) {
            throw new SyncFactoryException("IllegalAccessException: " + ex2.getMessage());
        }
        catch (InstantiationException ex3) {
            throw new SyncFactoryException("InstantiationException: " + ex3.getMessage());
        }
        catch (ClassNotFoundException ex4) {
            throw new SyncFactoryException("ClassNotFoundException: " + ex4.getMessage());
        }
    }
    
    public static Enumeration<SyncProvider> getRegisteredProviders() throws SyncFactoryException {
        initMapIfNecessary();
        return SyncFactory.implementations.elements();
    }
    
    public static void setLogger(Logger rsLogger) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SyncFactory.SET_SYNCFACTORY_PERMISSION);
        }
        if (rsLogger == null) {
            throw new NullPointerException("You must provide a Logger");
        }
        SyncFactory.rsLogger = rsLogger;
        return;
    }
    
    public static void setLogger(Logger rsLogger, Level level) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SyncFactory.SET_SYNCFACTORY_PERMISSION);
        }
        if (rsLogger == null) {
            throw new NullPointerException("You must provide a Logger");
        }
        rsLogger.setLevel(level);
        SyncFactory.rsLogger = rsLogger;
        return;
    }
    
    public static Logger getLogger() throws SyncFactoryException {
        Logger rsLogger = SyncFactory.rsLogger;
        if (rsLogger == null) {
            throw new SyncFactoryException("(SyncFactory) : No logger has been set");
        }
        return rsLogger;
    }
    
    public static synchronized void setJNDIContext(Context ic) throws SyncFactoryException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SyncFactory.SET_SYNCFACTORY_PERMISSION);
        }
        if (ic == null) {
            throw new SyncFactoryException("Invalid JNDI context supplied");
        }
        SyncFactory.ic = ic;
        return;
    }
    
    private static synchronized void initJNDIContext() throws SyncFactoryException {
        if (SyncFactory.ic != null && !SyncFactory.lazyJNDICtxRefresh) {
            try {
                parseProperties(parseJNDIContext());
                SyncFactory.lazyJNDICtxRefresh = true;
            }
            catch (NamingException ex) {
                ex.printStackTrace();
                throw new SyncFactoryException("SPI: NamingException: " + ex.getExplanation());
            }
            catch (Exception ex2) {
                ex2.printStackTrace();
                throw new SyncFactoryException("SPI: Exception: " + ex2.getMessage());
            }
        }
    }
    
    private static Properties parseJNDIContext() throws NamingException {
        NamingEnumeration<Binding> listBindings = SyncFactory.ic.listBindings("");
        Properties properties = new Properties();
        enumerateBindings(listBindings, properties);
        return properties;
    }
    
    private static void enumerateBindings(NamingEnumeration<?> namingEnumeration, Properties properties) throws NamingException {
        int n = 0;
        try {
            while (namingEnumeration.hasMore()) {
                Binding binding = (Binding)namingEnumeration.next();
                String name = binding.getName();
                Object object = binding.getObject();
                if (!(SyncFactory.ic.lookup(name) instanceof Context) && SyncFactory.ic.lookup(name) instanceof SyncProvider) {
                    n = 1;
                }
                if (n == 0) {
                    continue;
                }
                properties.put("rowset.provider.classname", ((SyncProvider)object).getProviderID());
                n = 0;
            }
        }
        catch (NotContextException ex) {
            namingEnumeration.next();
            enumerateBindings(namingEnumeration, properties);
        }
    }
    
    SyncFactory(SyncFactory.1 privilegedAction) {
        this();
    }
    
    static {
        SyncFactory.ROWSET_PROPERTIES = "rowset.properties";
        SET_SYNCFACTORY_PERMISSION = new SQLPermission("setSyncFactory");
        SyncFactory.colon = ":";
        SyncFactory.strFileSep = "/";
        SyncFactory.debug = false;
        SyncFactory.providerImplIndex = 0;
        SyncFactory.lazyJNDICtxRefresh = false;
    }
    
    private static class SyncFactoryHolder
    {
        static final SyncFactory factory;
        
        static {
            factory = new SyncFactory();
        }
    }
}
